function [a,yhat,uhat] = L1splines(Data,Pars,p,U)
% P-splines regression using the L1-norm:
% fitting the p^th quantile
% INPUT: Data = two columns (x,y) with y = response
%        Pars = [xmin xmax nseg deg lambda]
%                computation of a B-splines basis of degree 'deg'
%                on a uniform grid with 'nseg' intervals between
%                'xmin' and 'xmax'.  Lambda is smoothness parameter.
%        p = p^th quantile
%        U = values used for prediction.
%
% OUTPUT: a = coefficients
%         yhat = estimated values for Y
%         uhat = predicted values for U

% Paul Eilers & Kaatje Bollaerts, june 2005


% Construct B-splines basis for estimation
B = bsplbase(Data(:,1),Pars);
n = size(B, 2);

% Construct B-splines basis for prediction
Bpred = bsplbase(U,Pars);

% Construct Penalty matrices
E = eye(n)
S = diff(E, 2);
lambda = Pars(1,5);

% Data augmentation
B1 = [B; sqrt(lambda) * S];
y1 = [Data(:,2); zeros(n - 2, 1)];
m = length(Data(:,2));
q = [p * ones(m, 1); 0.5 * ones(n - 2, 1)];
a = rq_fnm(B1, y1, q);

% prediction
yhat = B * a;
uhat = Bpred * a;        
