function [a,yhat,uhat] = L1splines_iso(Data,Pars,p,U) 

% Isotone regression using P-splines and the L1-norm:
% fitting the p^th quantile
% INPUT: Data = two columns (x,y) with y = response
%        Pars = [xmin xmax nseg deg lambda]
%                computation of a B-splines basis of degree 'deg'
%                on a uniform grid with 'nseg' intervals between
%                'xmin' and 'xmax'.  Lambda is smoothness parameter.
%         p = p^th quantile
%         u = values used for prediction.
%
% OUTPUT: a = coefficients
%         yhat = estimated values for Y
%         uhat = predicted values for u

% Kaatje Bollaerts & Paul Eilers, june 2005


% Construct B-splines basis for estimation
B = bsplbase(Data(:,1),Pars);
n = size(B, 2);       

% Construct B-splines basis for prediction 
Bpred = bsplbase(U,Pars);

% Construct Penalty matrices
E = eye(n)
S = diff(E, 2);
lambda = Pars(1,5);
M = diff(E,1);
kappa = 1e6;

% Data augmentation
B1 = [B; sqrt(lambda) * S; sqrt(kappa) * M];
y1 = [Data(:,2); zeros(n - 2, 1); zeros(n - 1, 1)];
q = [p * ones(length(Data(:,2)), 1); 0.5 * ones(n - 2, 1);(1-(1/10^15)) * ones(n - 1,1)];

% L1 P-splines regression
a = rq_fnm(B1, y1, q);

% Predictin
yhat = B * a;
uhat = Bpred * a;        
